/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.api;

import org.nd4j.linalg.dataset.api.DataSet;
import org.nd4j.linalg.dataset.api.MultiDataSet;

/**
 * A Neural Network is an instance of a {@link NeuralNetworkConfiguration}, that can be trained,
 * evaluated, saved, exported, etc. Its configuration state is defined with the
 * {@link #setConfiguration(NeuralNetworkConfiguration)} and {@link #getConfiguration()} methods.
 *
 */
public interface NeuralNetwork {

  /**
   * The configuration that defines this Neural Network
   *
   * @param conf the configuration to use for this network
   */
  void setConfiguration(NeuralNetworkConfiguration conf);
  NeuralNetworkConfiguration getConfiguration();

  /**
   * This method fits model with a given DataSet
   *
   * @param dataSet the dataset to use for training
   */
  void fit(DataSet dataSet);

  /**
   * This method fits model with a given MultiDataSet
   *
   * @param dataSet the multi dataset to use for training
   */
  void fit(MultiDataSet dataSet);

  /**
   * The name of the Neural Network
   * @return the name
   */
  String getName();

  /**
   * Set the name for this Neural Network
   * @param name the name
   */
  void setName(String name);

}
