/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.api;

public class Animal {

  private String animalString;

  protected Animal(AnimalBuilder<?, ?> b) {
    this.animalString = b.animalString;
  }

  public static AnimalBuilder<?, ?> builder() {
    return new AnimalBuilderImpl();
  }

  public static abstract class AnimalBuilder<C extends Animal, B extends AnimalBuilder<C, B>> {

    private String animalString;

    public B animalString(String animalString) {
      this.animalString = animalString;
      return self();
    }

    protected abstract B self();

    public abstract C build();

    public String toString() {
      return "Animal.AnimalBuilder(animalString=" + this.animalString + ")";
    }
  }

  private static final class AnimalBuilderImpl extends
      AnimalBuilder<Animal, AnimalBuilderImpl> {

    private AnimalBuilderImpl() {
    }

    protected AnimalBuilderImpl self() {
      return this;
    }

    public Animal build() {
      return new Animal(this);
    }
  }
}
