/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.api;

/**
 * Activation Function An activation function takes in weighted data (matrix multiplication between
 * input data and weights) and outputs a non-linear transformation of the data. For example, output
 * = max(0,weighted_data) is the rectified linear activation function (essentially set all negative
 * values to zero). The difference between units and activation functions is that units can be more
 * complex, that is, a unit can have multiple activation functions (for example LSTM units) or a
 * slightly more complex structure (for example maxout units).
 * <p>
 * The difference between linear and non-linear activation functions can be shown with the
 * relationship of some weighted values: Imagine the four points A1, A2, B1 and B2. The pairs A1 /
 * A2, and B1 / B2 lie close to each other, but A1 is distant from B1 and B2, and vice versa; the
 * same for A2.
 * <p>
 * With a linear transformation the relationship between pairs might change. For example A1 and A2
 * might be far apart, but this implies that B1 and B2 are also far apart. The distance between the
 * pairs might shrink, but if it does, then both B1 and B2 will be close to A1 and A2 at the same
 * time. We can apply many linear transformations, but the relationship between A1 / A2 and B1 / B2
 * will always be similar.
 * <p>
 * In contrast, with a non-linear activation function we can increase the distance between A1 and A2
 * while we decrease the distance between B1 and B2. We can make B1 close to A1, but B2 distant from
 * A1. By applying non-linear functions, we create new relationships between the points. With every
 * new non-linear transformation we can increase the complexity of the relationships. In deep
 * learning, using non-linear activation functions creates increasingly complex features with every
 * layer.
 * <p>
 * In contrast, the features of 1000 layers of pure linear transformations can be reproduced by a
 * single layer (because a chain of matrix multiplication can always be represented by a single
 * matrix multiplication). This is why non-linear activation functions are so important in deep
 * learning.
 */
public interface IActivationFunction {

}
