/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.api;

import lombok.Getter;
import org.deeplearning4j.nn.conf.layers.ActivationLayer;
import org.deeplearning4j.nn.conf.layers.BatchNormalization;
import org.deeplearning4j.nn.conf.layers.ConvolutionLayer;
import org.deeplearning4j.nn.conf.layers.DropoutLayer;
import org.deeplearning4j.nn.conf.layers.FeedForwardLayer;
import org.deeplearning4j.nn.conf.layers.LayerConfiguration;
import org.deeplearning4j.nn.conf.layers.NoParamLayer;

public enum LayerType {
  CONV("CONV", "Convolutional", ConvolutionLayer.class),
  ACT("ACT", "Activation", ActivationLayer.class),
  POOL( "POOL", "Pooling/ Subsampling", NoParamLayer.class),
  FC( "FC", "Fully Connected", FeedForwardLayer.class),
  BN("BN", "Batch Normalization", BatchNormalization.class),
  DO("DO", "Dropout", DropoutLayer.class),
  UNKNOWN("UNKNOWN", "Type not specified", LayerConfiguration.class);

@Getter
  String description;
  @Getter String name;
  @Getter Class<? extends LayerConfiguration> clazz;

  LayerType(String name, String description, Class<? extends LayerConfiguration> clazz) {
    this.name = name;
    this.description = description;
    this.clazz = clazz;
  }
}
