/*
 *  ******************************************************************************
 *  *
 *  *
 *  * This program and the accompanying materials are made available under the
 *  * terms of the Apache License, Version 2.0 which is available at
 *  * https://www.apache.org/licenses/LICENSE-2.0.
 *  *
 *  *  See the NOTICE file distributed with this work for additional
 *  *  information regarding copyright ownership.
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  * License for the specific language governing permissions and limitations
 *  * under the License.
 *  *
 *  * SPDX-License-Identifier: Apache-2.0
 *  *****************************************************************************
 */

package org.deeplearning4j.nn.params;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import org.deeplearning4j.nn.api.AbstractParamInitializer;
import org.deeplearning4j.nn.api.ParamInitializer;
import org.deeplearning4j.nn.conf.NeuralNetConfiguration;
import org.deeplearning4j.nn.conf.layers.LayerConfiguration;
import org.deeplearning4j.nn.conf.layers.misc.FrozenLayer;
import org.nd4j.linalg.api.ndarray.INDArray;

public class FrozenLayerParamInitializer extends AbstractParamInitializer {

  private static final FrozenLayerParamInitializer INSTANCE = new FrozenLayerParamInitializer();

  public static FrozenLayerParamInitializer getInstance() {
    return INSTANCE;
  }

  @Override
  public long numParams(LayerConfiguration layer) {
    FrozenLayer fl = (FrozenLayer) layer;
    ParamInitializer initializer = fl.getInnerConfiguration().initializer();
    return initializer.numParams(fl.getInnerConfiguration());
  }

  @Override
  public List<String> paramKeys(LayerConfiguration layer) {
    return Collections.emptyList();
  }

  @Override
  public List<String> weightKeys(LayerConfiguration layer) {
    return Collections.emptyList();
  }

  @Override
  public List<String> biasKeys(LayerConfiguration layer) {
    return Collections.emptyList();
  }

  @Override
  public boolean isWeightParam(LayerConfiguration layer, String key) {
    return false;
  }

  @Override
  public boolean isBiasParam(LayerConfiguration layer, String key) {
    return false;
  }

  @Override
  public Map<String, INDArray> init(LayerConfiguration conf, INDArray paramsView,
      boolean initializeParams) {
    FrozenLayer fl_conf = (FrozenLayer) conf;
    LayerConfiguration innerLayer = fl_conf.getInnerConfiguration();
    ParamInitializer initializer = innerLayer.initializer();
    fl_conf.setInnerConfiguration(innerLayer);
    Map<String, INDArray> m = initializer.init(conf, paramsView, initializeParams);
    return m;
  }

  @Override
  public Map<String, INDArray> getGradientsFromFlattened(LayerConfiguration conf,
      INDArray gradientView) {
    FrozenLayer fl = (FrozenLayer) conf;
    LayerConfiguration innerLayer = fl.getInnerConfiguration();
    ParamInitializer initializer = innerLayer.initializer();
    fl.setInnerConfiguration(innerLayer);
    Map<String, INDArray> m = initializer.getGradientsFromFlattened(conf, gradientView);
    return m;
  }
}
